#!/bin/bash

CONF="../config/config.yaml"
CERTDIR="../config/certs"
TBSCRIPT="../config/db/tbscript.sh"
PGCREDS="065-secret-pgcreds"
YMLDIR="../yaml"
EXT1="key crt csr"
EXT2="key crt srl"

if [ "x$1" = "x--help" ]; then
        echo "Name: setupDB.sh"
        echo "When to run: Never, called automatically by the installer."
        echo "Description:"
        echo "   Configuration script called automatically by the installer.  It should only be called directly when troubleshooting."
        echo "   It will setup the Postgres container."
        echo "Example usage:"
        echo "   $ ./setupDB.sh"
        echo ""
        exit 0
fi

CDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
TDIR=$(basename $CDIR)
if [ "x$TDIR"  = "xsys" ]; then
	cd $CDIR/..
fi

kubectl=$(./sys/preReqCheck.sh)
RC=$(echo $?)
if [ $RC -ne 0 ]; then
	exit $RC
fi

NS=$(./sys/getNamespace.sh)
if [ $(echo $?) -ne 0 ]; then
	if [ "x$1" = "x" ]; then
		echo "Unable to determine namespace in use from values.yaml file"
	fi
	exit 7
fi

POD=$($kubectl -n $NS get pods | grep isvgimconfig | awk '{ print $1 }')
if [ "x$POD" = "x" ]; then
	./sys/startConfigContainer.sh
	sleep 5
	POD=$($kubectl -n $NS get pods | grep isvgimconfig | awk '{ print $1 }')
fi

# Make sure yaml dir exists
if [ ! -d $YMLDIR ]; then
	mkdir $YMLDIR
fi

# Setup SSL
POD=$($kubectl -n $NS get pods | grep isvgimconfig | grep Running | awk '{ print $1 }')
if [ -f $CERTDIR/isvgimRootCA.key ]; then
	for EXT in $EXT2; do
		$kubectl -n $NS cp $CERTDIR/isvgimRootCA.$EXT $POD:/work/isvgimRootCA.$EXT
	done
fi
$kubectl -n $NS exec $POD -- /work/certificateUtil.sh db
if [ $(echo $?) -ne 0 ]; then
	if [ "x$1" = "x" ]; then
		echo "Unable to create certificates in config container"
	fi
	exit 24
fi

for FILE in pgsql isvgimRootCA; do
	if [ $FILE = isvgimRootCA ]; then
		EXTS=$EXT2
	else
		EXTS=$EXT1
	fi
	for EXT in $EXTS; do
		$kubectl -n $NS cp $POD:/work/${FILE}.${EXT} ${CERTDIR}/${FILE}.${EXT} > /dev/null 2>&1
	done
done

# Generate configmap for certs
./createConfigs.sh db
RC=$(echo $?)
if [ $RC -ne 0 ]; then
	if [ "x$1" = "x" ]; then
		echo "Error creating ConfigMap files.  Please review above messages for details"
	fi
	exit $RC
fi

# Verify user and password are filled out
PGUSER=$(grep "  user:" $CONF | awk '{ print $2 }')
if [ "x$PGUSER" = "x" ]; then
	if [ "x$1" = "x" ]; then
		echo "You must specify a user for DB in config.yaml"
	fi
	exit 19
fi

# Add Quotes to username to preserve the case if it exists.
PGUSER=\"$PGUSER\"

PGPASS=$(grep "password:" $CONF | awk '{ print $2 }')
if [ "x$PGPASS" = "x" ]; then
	if [ "x$1" = "x" ]; then
		echo "You must specify a password for DB in config.yaml"
	fi
	exit 21
fi

# Turn credentials into a secret
$kubectl create secret generic pgcreds --from-literal=pguser=$PGUSER --from-literal=pgpass=$PGPASS --from-literal=admpass=$ADMPASS --from-literal=admuser=$ADMUSER --dry-run=client -o yaml --namespace=isvg-temp | sed 's/isvg-temp/{{ .Values.namespace }}/g' > ../helm/templates/${PGCREDS}.yaml
if [ $(echo $?) -ne 0 ]; then
	if [ "x$1" = "x" ]; then
		echo "Error creating pgcreds secret"
	fi
	exit 22
fi
./updateYaml.sh ${PGCREDS}.yaml

# Create tablespace dir script
cat <<EOF > $TBSCRIPT
cd /var/lib/postgresql/data
mkdir isvgimdata
mkdir isvgimindexes
EOF

deploy_single() {
	YMLFILES="040-config-isvgimdb 080-pvc-postgres 130-service-postgres 220-deployment-postgres"
	for FILE in $YMLFILES; do
		./updateYaml.sh ${FILE}.yaml
	done

	# Waiting for pod to be created
	sleep 5
	POD=$($kubectl -n $NS get pods | grep postgres | awk '{ print $1 }')
	./sys/waitFor.sh $POD pod
	RC=$(echo $?)
	if [ $RC -ne 0 ]; then
		exit $RC
	fi

	# Waiting for replica-1 pod to be ready
	./sys/waitFor.sh $POD application
	RC=$(echo $?)
	if [ $RC -ne 0 ]; then
		exit $RC
	fi
	create_tablespace_dirs $POD
} # deploy_single

deploy_cluster() {
	YMLFILES="040-config-isvgimdb 405-operator-kubegres 410-crd-postgres"
	for FILE in $YMLFILES; do
		./updateYaml.sh ${FILE}.yaml
	done

	# Waiting for pod to be created
	sleep 5
	./sys/waitFor.sh postgres-3-0 pod
	RC=$(echo $?)
	if [ $RC -ne 0 ]; then
		exit $RC
	fi

	# Waiting for replica pods to be ready
	./sys/waitFor.sh postgres-3-0 application
	RC=$(echo $?)
	if [ $RC -ne 0 ]; then
		exit $RC
	fi

	# Create ISVGIM tablespace directories
	PGPODS="postgres-1-0 postgres-2-0 postgres-3-0"
	for POD in $PGPODS; do
		create_tablespace_dirs $POD
	done
} # deploy_cluster

create_tablespace_dirs() {
	RESULT=$($kubectl -n $NS cp $TBSCRIPT ${1}:/tmp/tbscript.sh 2>&1)
	RC=$(echo $?)
	if [ $RC -ne 0 ]; then
		echo "$RESULT"
		exit 43
	fi
	RESULT=$($kubectl -n $NS exec $1 -- /bin/bash /tmp/tbscript.sh 2>&1)
	RC=$(echo $?)
	if [ $RC -ne 0 ]; then
		echo "$RESULT"
		exit 44
	fi
} # create_tablespace_dirs

printf "\n  -- Deploying Postgres --\n"
if [ $INSTALL_DB_HA -eq 0 ]; then
	deploy_single
else
	deploy_cluster
fi
rm $TBSCRIPT
